# -*- coding: utf-8 -*-
"""coop_bar configuration: add links to balafon"""

from django.urls import reverse
from django.utils.translation import ugettext as _

from coop_bar.utils import make_link


def balafon_homepage(request, context):
    """back to balafon homepage"""
    if request and request.user.is_authenticated and request.user.is_staff:
        return make_link(
            reverse("balafon_homepage"),
            _('Balafon'),
            'address-book',
            classes=['icon', 'alert_on_click']
        )


def back_to_newsletters(request, context):
    """back to balafon newsletters"""
    if request and request.user.is_authenticated and request.user.is_staff:
        return make_link(
            reverse("emailing_newsletter_list"),
            _('Back to newsletters'),
            'arrow-left',
            classes=['icon', 'alert_on_click']
        )


def load_commands(coop_bar):
    """load commands"""
    coop_bar.register([
        [balafon_homepage, back_to_newsletters],
    ])



# -*- coding: utf-8 -*-
"""REST api powered by django-rest-framework"""

import re

from django.contrib.auth.models import User
from django.db.models import Q
from django.utils.translation import ugettext

from rest_framework import serializers

from balafon.Crm.models import (
    Action, ActionType, ActionStatus, Contact, City, Entity, EntityType, TeamMember, Zone
)
from balafon.Crm.settings import get_default_country


class TeamMemberSerializer(serializers.HyperlinkedModelSerializer):
    """Serializers define the API representation."""

    class Meta:
        model = TeamMember
        fields = ('id', 'name', )


class UserSerializer(serializers.HyperlinkedModelSerializer):
    """Serializers define the API representation."""

    class Meta:
        model = User
        fields = ('id', 'username', 'last_name', 'first_name', )


class CitySerializer(serializers.ModelSerializer):

    class Meta:
        model = City
        fields = ('id', 'name')


class CountrySerializer(serializers.ModelSerializer):
    is_foreign_country = serializers.BooleanField(read_only=True)

    class Meta:
        model = Zone
        fields = ('id', 'name', 'is_foreign_country', )


class ContactSerializer(serializers.ModelSerializer):
    """Serialize a contact"""
    get_city = CitySerializer(read_only=True)
    get_country = CountrySerializer(read_only=True)
    get_address = serializers.CharField(read_only=True)
    get_address2 = serializers.CharField(read_only=True)
    get_address3 = serializers.CharField(read_only=True)
    get_zip_code = serializers.CharField(read_only=True)
    get_cedex = serializers.CharField(read_only=True)
    get_billing_city = CitySerializer(read_only=True)
    get_billing_country = CountrySerializer(read_only=True)
    get_billing_address = serializers.CharField(read_only=True)
    get_billing_address2 = serializers.CharField(read_only=True)
    get_billing_address3 = serializers.CharField(read_only=True)
    get_billing_zip_code = serializers.CharField(read_only=True)
    get_billing_cedex = serializers.CharField(read_only=True)
    get_phone = serializers.CharField(read_only=True)
    get_view_url = serializers.CharField(read_only=True)

    class Meta:
        model = Contact
        fields = (
            'id', 'fullname', 'lastname', 'firstname', 'title', 'job',
            'get_city', 'get_address', 'get_address2', 'get_address3', 'get_cedex', 'get_zip_code', 'get_country',
            'get_billing_city', 'get_billing_address', 'get_billing_address2', 'get_billing_address3',
            'get_billing_cedex', 'get_billing_zip_code', 'get_billing_country',
            'mobile', 'get_phone', 'email', 'notes', 'get_view_url', 'favorite_language',
        )


class NewContactSerializer(serializers.ModelSerializer):
    """Write a new contact"""
    lastname = serializers.CharField(required=True)
    firstname = serializers.CharField(required=True)
    address = serializers.CharField(required=True)
    zip_code = serializers.CharField(required=True)
    city = serializers.CharField(required=True)
    country = serializers.IntegerField(required=True)
    email = serializers.EmailField(required=True)

    class Meta:
        model = Contact
        fields = (
            'lastname', 'firstname', 'email', 'address', 'address2', 'address3', 'cedex', 'zip_code',
            'country', 'city'
        )

    def validate_country(self, country_id):
        try:
            return Zone.objects.get(id=country_id, parent__isnull=True)
        except Zone.DoesNotExist:
            raise serializers.ValidationError(ugettext('Invalid country'))

    def validate_zip_code(self, zip_code):
        return zip_code.strip()

    def validate_city(self, city_name):
        words = re.split(r'\W+', city_name)
        city_name = '-'.join([word.capitalize() for word in words if word])
        if not city_name:
            raise serializers.ValidationError(ugettext('The city name is required'))
        return city_name

    def get_city(self, validated_data):
        zip_code = validated_data['zip_code']
        city_name = validated_data['city']
        country = validated_data.pop('country')
        default_country = get_default_country()
        if country.name != default_country:
            city = City.objects.get_or_create(name=city_name, parent=country)[0]
        else:
            if len(zip_code) < 2:
                raise serializers.ValidationError(ugettext('You must enter a zip code'))
            try:
                dep = Zone.objects.get(Q(code=zip_code[:2]) | Q(code=zip_code[:3]))
            except Zone.DoesNotExist:
                raise serializers.ValidationError(ugettext('Invalid zip code'))
            city = City.objects.get_or_create(name=city_name, parent=dep)[0]
        return city

    def validate(self, attrs):
        validated_data = super(NewContactSerializer, self).validate(attrs)
        validated_data['city'] = self.get_city(validated_data)
        return validated_data


class EntityTypeSerializer(serializers.ModelSerializer):
    """Serialize an entity type"""

    class Meta:
        model = EntityType
        fields = ('id', 'name',)


class EntitySerializer(serializers.ModelSerializer):
    """Serialize an entity"""
    city = CitySerializer(read_only=True)
    type = EntityTypeSerializer(read_only=True)
    get_view_url = serializers.CharField(read_only=True)
    country = CountrySerializer(read_only=True)
    billing_city = CitySerializer(read_only=True)
    billing_country = CountrySerializer(read_only=True)

    class Meta:
        model = Entity
        fields = (
            'id', 'name', 'type', 'city', 'address', 'address2', 'address3', 'cedex', 'country', 'zip_code',
            'billing_city', 'billing_address', 'billing_address2', 'billing_address3', 'billing_cedex',
            'billing_zip_code', 'billing_country', 'phone', 'email', 'website', 'notes', 'get_view_url',
        )


class ActionStatusSerializer(serializers.ModelSerializer):
    """action status"""

    class Meta:
        model = ActionStatus
        fields = ('id', 'name')


class ActionTypeSerializer(serializers.ModelSerializer):
    """action type serializer"""
    allowed_status = ActionStatusSerializer(many=True, read_only=True)
    default_status = ActionStatusSerializer(read_only=True)
    default_status2 = ActionStatusSerializer(read_only=True)

    class Meta:
        model = ActionType
        fields = (
            'id', 'name', 'allowed_status', 'default_status', 'allowed_status2', 'default_status2', 'order_index',
        )


class ActionSerializer(serializers.ModelSerializer):
    """Serialize an action"""
    contacts = ContactSerializer(many=True, read_only=True)
    entities = EntitySerializer(many=True, read_only=True)
    last_modified_by = UserSerializer(many=False, read_only=True)
    mail_to = serializers.CharField(read_only=True)

    class Meta:
        model = Action
        fields = (
            'id', 'contacts', 'entities', 'subject', 'planned_date', 'end_datetime', 'type', 'status', 'status2',
            'in_charge', 'detail', 'last_modified_by', 'modified', 'get_action_number', 'mail_to'
        )


class ActionUpdateDateSerializer(serializers.ModelSerializer):
    """Serialize date for updating action date"""

    class Meta:
        model = Action
        fields = ('planned_date', 'end_datetime',)


class ContactIdSerializer(serializers.ModelSerializer):
    """Serialize contact Id"""

    class Meta:
        model = Contact
        fields = ('id', )


class ActionUpdateSerializer(serializers.ModelSerializer):
    """Serialize data for updating an action"""
    contacts = ContactIdSerializer(many=True, read_only=True)
    planned_date = serializers.DateTimeField(allow_null=True)
    end_datetime = serializers.DateTimeField(allow_null=True)

    class Meta:
        model = Action
        fields = (
            'contacts', 'planned_date', 'end_datetime', 'subject', 'in_charge', 'detail', 'type', 'status', 'status2',
        )



# -*- coding: utf-8 -*-
"""common utils"""

from bs4 import BeautifulSoup
from datetime import datetime
import logging
from urllib.parse import urlparse
from importlib import import_module

from django.conf import settings
from django.contrib.sites.models import Site
from django.core.mail import get_connection, EmailMultiAlternatives
from django.core.exceptions import ValidationError
from django.http import HttpResponseRedirect, Http404
from django.template.loader import get_template
from django.urls import resolve, Resolver404
from django.utils.translation import ugettext as _

from coop_cms.utils.requests import RequestManager
from coop_cms.utils import dehtml
from rest_framework.renderers import JSONRenderer

from balafon.settings import get_allowed_homepages


logger = logging.getLogger("balafon_crm")


def log_error(view_func):
    """log error decorator"""
    def wrapper(request, *args, **kwargs):
        """wrapper"""
        try:
            return view_func(request, *args, **kwargs)
        except Http404:
            raise
        except:
            logger.exception("exception")
            raise
    return wrapper


def now_rounded():
    """datetime rounded to the minute"""
    now = datetime.now()
    return datetime(now.year, now.month, now.day, now.hour, now.minute, 0, 0)


class HttpResponseRedirectMailtoAllowed(HttpResponseRedirect):
    """Mailto redirect"""
    allowed_schemes = ['http', 'https', 'ftp', 'mailto']


def get_form_errors(response):
    """get form errors"""
    soup = BeautifulSoup(response.content, "html.parser")
    errors = soup.select('.field-error .label')
    return errors


def is_allowed_homepage(url_string):
    """return True is the current page can be set as homepage"""
    url = urlparse(url_string)
    try:
        safe_url = url.path
        resolved = resolve(safe_url)
    except Resolver404:
        return False
    if resolved.url_name in get_allowed_homepages():
        return True
    return False


def validate_rgb(value):
    """"check rgb"""
    wrong = False
    if not (len(value) == 7 or len(value) == 4):
        wrong = True
    else:
        if value[0] != "#":
            wrong = True
        else:
            try:
                int(value[1:], 16)
            except ValueError:
                wrong = True
    if wrong:
        raise ValidationError(_('RGB format (e.g. #123456) expected'))
    return False


class Utf8JSONRenderer(JSONRenderer):
    """Utf-8 support"""
    def render(self, *args, **kwargs):
        content = super(Utf8JSONRenderer, self).render(*args, **kwargs)
        return content.decode('utf-8')


def load_from_module(settings_key, default_value):
    """returns the form to be used for creating a new article"""
    full_class_name = getattr(settings, settings_key, '') or default_value
    if full_class_name:
        try:
            module_name, obj_name = full_class_name.rsplit('.', 1)
        except ValueError:
            raise ImportError("Unable to import {0}: full path is required".format(full_class_name))
        module = import_module(module_name)
        obj = getattr(module, obj_name)
        return obj
    return None


def send_email(subject, template_name, context, recipients, sender=None, cc_list=None):
    """Send an HTML email"""
    emails = []
    connection = get_connection()
    from_email = sender if sender else settings.DEFAULT_FROM_EMAIL
    for address in recipients:
        context['email_address'] = address
        context['subject'] = subject
        the_template = get_template(template_name)
        html_text = the_template.render(context)
        text = dehtml(html_text)
        email = EmailMultiAlternatives(subject, text, from_email, [address], cc=cc_list if cc_list else [])
        email.attach_alternative(html_text, "text/html")
        emails.append(email)
    return connection.send_messages(emails)


def full_path_url(url):
    request = RequestManager().get_request()
    is_secured = not settings.DEBUG
    if request:
        is_secured = request.is_secure()
    site = Site.objects.get_current(request)
    if url is None:
        url = ''
    full_url = site.domain + url
    if is_secured:
        return 'https://' + full_url
    else:
        return 'http://' + full_url



# Generated by Django 2.2.17 on 2020-11-23 10:25

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('Profile', '0005_auto_20180409_1301'),
    ]

    operations = [
        migrations.CreateModel(
            name='ContactProfileCustomField',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100, verbose_name='name')),
                ('value', models.CharField(blank=True, default='', max_length=200, verbose_name='value')),
                ('entity_field', models.BooleanField(default=False)),
                ('profile', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='custom_fields', to='Profile.ContactProfile')),
            ],
        ),
    ]



# Generated by Django 2.2.9 on 2020-02-10 14:36

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Users', '0005_auto_20190719_0947'),
    ]

    operations = [
        migrations.AlterField(
            model_name='custommenuitem',
            name='only_for_users',
            field=models.ManyToManyField(blank=True, limit_choices_to={'is_staff': True}, to=settings.AUTH_USER_MODEL, verbose_name='only for users'),
        ),
    ]



# -*- coding: utf-8 -*-

from django.contrib import admin
from balafon.Users import models
from balafon.Users.forms import UserPreferencesAdminForm, UserPermissionsAdminForm


@admin.register(models.UserPreferences)
class UserPreferencesAdmin(admin.ModelAdmin):
    form = UserPreferencesAdminForm
    list_display = ['user', 'notify_due_actions', 'message_in_favorites', ]


@admin.register(models.UserPermissions)
class UserPermissionsAdmin(admin.ModelAdmin):
    form = UserPermissionsAdminForm
    list_display = ['user', 'can_create_group', ]
    list_editable = ['can_create_group', ]


@admin.register(models.Favorite)
class FavoriteAdmin(admin.ModelAdmin):
    list_display = ["__str__", "user", "content_type", "object_id"]
    list_filter = ["user", "content_type"]


@admin.register(models.UserHomepage)
class UserHomepageAdmin(admin.ModelAdmin):
    list_display = ["user", "url"]


class CustomMenuItemInline(admin.StackedInline):
    model = models.CustomMenuItem
    fields = [
        'label', 'icon', 'url', 'reverse', 'reverse_kwargs', 'attributes',
        'query_string', 'order_index', 'only_for_users'
    ]
    raw_id_fields = ['only_for_users']
    extra = 0


@admin.register(models.CustomMenu)
class CustomMenuAdmin(admin.ModelAdmin):
    inlines = [CustomMenuItemInline]



# Generated by Django 2.2.17 on 2020-12-02 17:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Crm', '0030_group_on_contacts_list'),
    ]

    operations = [
        migrations.AddField(
            model_name='actionmenu',
            name='css_class',
            field=models.CharField(blank=True, default='', help_text='Example: "colorbox-form" for colorbox display', max_length=50, verbose_name='CSS class'),
        ),
        migrations.AlterField(
            model_name='actionmenu',
            name='a_attrs',
            field=models.CharField(blank=True, default='', help_text='Example: target="_blank"', max_length=50, verbose_name='Link args'),
        ),
    ]



# Generated by Django 2.2.9 on 2020-02-17 15:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Crm', '0029_subscriptiontype_allowed_on_sites'),
    ]

    operations = [
        migrations.AddField(
            model_name='group',
            name='on_contacts_list',
            field=models.BooleanField(default=False, verbose_name='Displayed on contacts list'),
        ),
    ]



# Generated by Django 2.2.17 on 2020-11-27 14:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Store', '0020_sale_delivery_details'),
    ]

    operations = [
        migrations.AddField(
            model_name='saleitem',
            name='is_extra',
            field=models.BooleanField(default=False, help_text='added after total on the bill', verbose_name='is extra'),
        ),
    ]



# -*- coding: utf-8 -*-

from django.contrib.admin.widgets import ManyToManyRawIdWidget
from django.urls import reverse
from django.utils.encoding import smart_text
from django.utils.html import escape, mark_safe

import floppyforms.__future__ as forms


class VerboseManyToManyRawIdWidget(ManyToManyRawIdWidget):
    """
    A Widget for displaying ManyToMany ids in the "raw_id" interface rather than
    in a <select multiple> box. Display user-friendly value like the ForeignKeyRawId widget
    """

    def __init__(self, rel, attrs=None, *args, **kwargs):
        super().__init__(rel, attrs, *args, **kwargs)

    def label_and_url_for_value(self, value):
        values = value
        str_values = []
        key = self.rel.field.name
        app_label = self.rel.model._meta.app_label
        class_name = self.rel.model._meta.model_name.lower()
        for elt in values:
            try:
                obj = self.rel.related_model.objects.using(self.db).get(**{key: elt})
                url = reverse('admin:{0}_{1}_change'.format(app_label, class_name), args=[obj.id])
                label = escape(smart_text(obj))
                link = '<a href="{0}" {1}>{2}</a>'.format(
                    url,
                    'onclick="return showAddAnotherPopup(this);" target="_blank"',
                    label
                )
                str_values += [link]
            except (self.rel.related_model.DoesNotExist, self.rel.related_model.MultipleObjectsReturned):
                str_values += ['???']
        return mark_safe('&nbsp;<strong>{0}</strong>'.format(',&nbsp;'.join(str_values))), ''


class CalcHiddenInput(forms.HiddenInput):
    """This field is calculated and hidden. It is not in POST data"""

    def value_omitted_from_data(self, data, files, name):
        # Tell django that this is field value doesn't come from POST data
        return False



# Generated by Django 2.2.18 on 2021-05-13 17:10

from django.db import migrations, models
import django.db.models.manager


class Migration(migrations.Migration):

    dependencies = [
        ('Crm', '0032_auto_20210223_0940'),
    ]

    operations = [
        migrations.AlterModelManagers(
            name='contact',
            managers=[
                ('all_objects', django.db.models.manager.Manager()),
            ],
        ),
        migrations.AlterModelManagers(
            name='entity',
            managers=[
                ('all_objects', django.db.models.manager.Manager()),
            ],
        ),
        migrations.AddField(
            model_name='contact',
            name='confirmed',
            field=models.BooleanField(db_index=True, default=True, help_text='For contact created from subscription form, we need to wait for confirmation by email link', verbose_name='confirmed'),
        ),
        migrations.AddField(
            model_name='entity',
            name='confirmed',
            field=models.BooleanField(db_index=True, default=True, help_text='For contact created from subscription form, we need to wait for confirmation by email link', verbose_name='confirmed'),
        ),
    ]



# -*- coding: utf-8 -*-
"""common place for all Emailing settings"""

from django.conf import settings as project_settings


def is_registration_enabled():
    """return true if subscribe page is enabled : True by default for compatibility reason"""
    return getattr(project_settings, 'BALAFON_REGISTRATION_ENABLED', True)


def is_html_activation_email():
    """If True : use custom send_email and send by HTML"""
    return getattr(project_settings, 'BALAFON_ACTIVATION_EMAIL_HTML', False)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-07-19 09:47
from __future__ import unicode_literals

from django.db import migrations


def change_glyph_into_fontawesome(apps, schema_editor):
    """Assign the Internet AalysisCode to all existing sales"""
    custom_menu_class = apps.get_model("Users", "CustomMenu")
    custom_menu_item_class = apps.get_model("Users", "CustomMenuItem")
    
    try:
        custom_menu_compta = custom_menu_class.objects.get(label="Compta")
        custom_menu_compta_item = custom_menu_item_class.objects.get(parent=custom_menu_compta)

        if custom_menu_compta_item.icon == "euro":
            custom_menu_compta_item.icon = "euro-sign"
            custom_menu_compta_item.save()
            
    except:
        pass
    
    try:
        custom_menu_imprimer = custom_menu_class.objects.get(label="Imprimer")
        custom_menu_imprimer_item = custom_menu_item_class.objects.filter(parent=custom_menu_imprimer)
    
        for item in custom_menu_imprimer_item:
            if item.icon == "euro":
                item.icon = "euro-sign"
                item.save()
                
    except:
        pass

    try:
        custom_menu_epicerie = custom_menu_class.objects.get(label="Epicerie")
        custom_menu_epicerie_item = custom_menu_item_class.objects.filter(parent=custom_menu_epicerie)
    
        for item in custom_menu_epicerie_item:
            if item.icon == "euro":
                item.icon = "euro-sign"
                item.save()

    except:
        pass
    
    try:
        custom_menu_boulangerie = custom_menu_class.objects.get(label="Boulangerie")
        custom_menu_boulangerie_item = custom_menu_item_class.objects.filter(parent=custom_menu_boulangerie)
    
        for item in custom_menu_boulangerie_item:
            if item.icon == "euro":
                item.icon = "euro-sign"
                item.save()
        
            elif item.icon == "grain":
                item.icon = "wheat"
                item.save()

    except:
        pass
    
    try:
        custom_menu_fruits = custom_menu_class.objects.get(label="Fruits")
        custom_menu_fruits_item = custom_menu_item_class.objects.filter(parent=custom_menu_fruits)
    
        for item in custom_menu_fruits_item:
            if item.icon == "apple":
                item.icon = "apple-alt"
                item.save()
                
    except:
        pass

def change_fontawesome_into_glyph(apps, schema_editor):
    """Assign the Internet AalysisCode to all existing sales"""
    custom_menu_class = apps.get_model("Users", "CustomMenu")
    custom_menu_item_class = apps.get_model("Users", "CustomMenuItem")
    
    try:
        custom_menu_compta = custom_menu_class.objects.get(label="Compta")
        custom_menu_compta_item = custom_menu_item_class.objects.get(parent=custom_menu_compta)
    
        if custom_menu_compta_item.icon == "euro-sign":
            custom_menu_compta_item.icon = "euro"
            custom_menu_compta_item.save()

    except:
        pass

    try:
        custom_menu_imprimer = custom_menu_class.objects.get(label="Imprimer")
        custom_menu_imprimer_item = custom_menu_item_class.objects.filter(parent=custom_menu_imprimer)
        
        for item in custom_menu_imprimer_item:
            if item.icon == "euro-sign":
                item.icon = "euro"
                item.save()

    except:
        pass

    try:
        custom_menu_epicerie = custom_menu_class.objects.get(label="Epicerie")
        custom_menu_epicerie_item = custom_menu_item_class.objects.filter(parent=custom_menu_epicerie)
        
        for item in custom_menu_epicerie_item:
            if item.icon == "euro-sign":
                item.icon = "euro"
                item.save()

    except:
        pass

    try:
        custom_menu_boulangerie = custom_menu_class.objects.get(label="Boulangerie")
        custom_menu_boulangerie_item = custom_menu_item_class.objects.filter(parent=custom_menu_boulangerie)
    
        for item in custom_menu_boulangerie_item:
            if item.icon == "euro-sign":
                item.icon = "euro"
                item.save()
        
            elif item.icon == "wheat":
                item.icon = "grain"
                item.save()

    except:
        pass

    try:
        custom_menu_fruits = custom_menu_class.objects.get(label="Fruits")
        custom_menu_fruits_item = custom_menu_item_class.objects.filter(parent=custom_menu_fruits)
        
        for item in custom_menu_fruits_item:
            if item.icon == "apple-alt":
                item.icon = "apple"
                item.save()

    except:
        pass

class Migration(migrations.Migration):

    dependencies = [
        ('Users', '0004_userpermissions'),
    ]

    operations = [
        migrations.RunPython(change_glyph_into_fontawesome, change_fontawesome_into_glyph),
    ]



# -*- coding: utf-8 -*-
"""permission rules and utilities"""

def can_access(user):
    """define who can access"""
    return user.is_authenticated and user.is_staff


def is_admin(user):
    """define who can access"""
    return user.is_authenticated and user.is_superuser



# Generated by Django 2.2.17 on 2020-11-26 21:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Store', '0018_sale_details'),
    ]

    operations = [
        migrations.CreateModel(
            name='Voucher',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(max_length=100, verbose_name='code')),
                ('start_date', models.DateField(verbose_name='start date')),
                ('end_date', models.DateField(verbose_name='end date')),
                ('rate', models.DecimalField(decimal_places=2, default=0, max_digits=4, verbose_name='rate')),
                ('active', models.BooleanField(default=False, verbose_name='active')),
            ],
            options={
                'verbose_name': 'Voucher',
                'verbose_name_plural': 'Vouchers',
                'ordering': ['-start_date', 'end_date'],
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-03-28 13:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Crm', '0026_auto_20180924_1059'),
    ]

    operations = [
        migrations.AlterField(
            model_name='contact',
            name='favorite_language',
            field=models.CharField(blank=True, default='', max_length=10, verbose_name='favorite language'),
        ),
    ]



# -*- coding: utf-8 -*-
"""common place for settings. Define defaults"""

from django.conf import settings
from django.utils.translation import ugettext as _

from ..utils import load_from_module


def is_public_api_allowed():
    """returns true if an anonymous can view some data : products, categories, tags"""
    return getattr(settings, 'BALAFON_STORE_ALLOW_PUBLIC_API', False)


def get_cart_type_name():
    """returns true if an anonymous can view some data : products, categories, tags"""
    return getattr(settings, 'BALAFON_STORE_CART_TYPE', _('Cart'))


def get_cart_confirmation_subject(profile, action):
    """returns true if an anonymous can view some data : products, categories, tags"""
    subject_callback = load_from_module('BALAFON_CART_CONFIRMATION_EMAIL_SUBJECT_CALLBACK', None)
    if subject_callback is None:
        subject = getattr(settings, 'BALAFON_CART_CONFIRMATION_EMAIL_SUBJECT', _('Purchasing confirmation'))
        if callable(subject):
            return subject(profile, action)
        else:
            return subject
    else:
        return subject_callback(profile, action)


def get_cart_processed_callback():
    """
    returns a callback defined in BALAFON_CART_PROCESSED_CALLBACK
    This function should take an action has argument
    """
    return load_from_module('BALAFON_CART_PROCESSED_CALLBACK', None)


def get_notify_cart_callback():
    """
    returns a callback defined in BALAFON_NOTIFY_CART_CALLBACK
    This function should take an action has argument
    If not defined : notify to user and manager by email
    """
    return load_from_module('BALAFON_NOTIFY_CART_CALLBACK', None)


def get_thumbnail_size():
    """
    returns size of image when displayed as thumbnail
    """
    return getattr(settings, 'BALAFON_STORE_THUMBNAIL_SIZE', "64x64")


def get_thumbnail_crop():
    """
    returns size of image when displayed as thumbnail
    """
    return getattr(settings, 'BALAFON_STORE_THUMBNAIL_CROP', "center")


def get_image_size():
    """
    returns size of image when displayed as thumbnail
    """
    return getattr(settings, 'BALAFON_STORE_IMAGE_SIZE', "300")


def get_image_crop():
    """
    returns size of image when displayed as thumbnail
    """
    return getattr(settings, 'BALAFON_STORE_IMAGE_CROP', None)


def get_commercial_document_pdf_options():
    return getattr(settings, 'BALAFON_COMMERCIAL_DOCUMENT_WKHTMLTOPDF_OPTIONS', None)


def allow_cart_no_profile():
    return getattr(settings, 'BALAFON_ALLOW_CART_NO_PROFILE', False)



# -*- coding: utf-8 -*-
"""unit testing"""

from datetime import datetime
import logging

from django.conf import settings
from django.core import mail
from django.test.utils import override_settings
from django.urls import reverse

from model_mommy import mommy
from rest_framework import status
from rest_framework.test import APITestCase, APIClient

from balafon.Crm.models import Action, ActionType, Zone, ZoneType, Contact
from balafon.Crm.settings import get_default_country
from balafon.Store import models
from balafon.Store.settings import get_cart_type_name


@override_settings(BALAFON_ALLOW_CART_NO_PROFILE=True)
class CartTest(APITestCase):
    """Base class for test cases"""
    fixtures = ['zones.json']

    def setUp(self):
        super(CartTest, self).setUp()
        logging.disable(logging.CRITICAL)
        self.client = APIClient()

    def tearDown(self):
        logging.disable(logging.NOTSET)
        super(CartTest, self).tearDown()

    def test_post_cart_no_profile(self):
        """It should create a new sale and action"""

        store_item1 = mommy.make(models.StoreItem)
        store_item2 = mommy.make(models.StoreItem)

        delivery_point = mommy.make(models.DeliveryPoint)

        zone_type = ZoneType.objects.get_or_create(type="country")[0]
        default_country = get_default_country()
        country = Zone.objects.get_or_create(name=default_country, type=zone_type, parent=None)[0]

        contact_data = {
            'lastname': 'Dupond',
            'firstname': 'Paul',
            'email': 'paul@dupond.fr',
            'address': '1 rue la ré',
            'city': 'Lyon',
            'zip_code': '69000',
            'country': country.id,
        }

        data = {
            'items': [
                {'id': store_item1.id, 'quantity': 2},
                {'id': store_item2.id, 'quantity': 1},
            ],
            'delivery_point': delivery_point.id,
            'purchase_datetime': datetime(2015, 7, 23, 12, 0),
            'contact': contact_data,
        }

        url = reverse('store_post_cart_no_profile')

        response = self.client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(response.data['ok'], True)
        self.assertEqual(len(response.data['warnings']), 0)
        self.assertEqual(response.data['deliveryDate'], data['purchase_datetime'])
        self.assertEqual(response.data['deliveryPlace'], delivery_point.name)

        action_type = ActionType.objects.get(name=get_cart_type_name())

        action_queryset = Action.objects.filter(type=action_type)
        self.assertEqual(action_queryset.count(), 1)
        action = action_queryset[0]

        self.assertEqual(response.data['action'], action.id)

        self.assertEqual(action.contacts.count(), 1)
        contact = action.contacts.all()[0]
        self.assertEqual(contact.lastname, contact_data['lastname'])
        self.assertEqual(contact.firstname, contact_data['firstname'])
        self.assertEqual(contact.email, contact_data['email'])
        self.assertEqual(contact.address, contact_data['address'])
        self.assertEqual(contact.city.name, contact_data['city'])
        self.assertEqual(contact.zip_code, contact_data['zip_code'])
        self.assertEqual(contact.get_country(), country)

        self.assertEqual(action.subject, '')
        self.assertEqual(action.detail, '')
        self.assertEqual(action.planned_date, data['purchase_datetime'])
        self.assertEqual(action.sale.payment_mode, None)
        self.assertEqual(action.sale.analysis_code, models.SaleAnalysisCode.objects.get(name='Internet'))

        self.assertEqual(action.sale.saleitem_set.count(), 2)
        self.assertEqual(action.sale.saleitem_set.all()[0].item, store_item1)
        self.assertEqual(action.sale.saleitem_set.all()[0].text, store_item1.name)
        self.assertEqual(action.sale.saleitem_set.all()[0].unit_price(), store_item1.pre_tax_price)
        self.assertEqual(action.sale.saleitem_set.all()[0].vat_rate, store_item1.vat_rate)
        self.assertEqual(action.sale.saleitem_set.all()[0].quantity, 2)

        self.assertEqual(action.sale.saleitem_set.all()[1].item, store_item2)
        self.assertEqual(action.sale.saleitem_set.all()[1].text, store_item2.name)
        self.assertEqual(action.sale.saleitem_set.all()[1].unit_price(), store_item2.pre_tax_price)
        self.assertEqual(action.sale.saleitem_set.all()[1].vat_rate, store_item2.vat_rate)
        self.assertEqual(action.sale.saleitem_set.all()[1].quantity, 1)

        self.assertEqual(len(mail.outbox), 2)
        self.assertEqual(mail.outbox[0].to, [contact.email])
        self.assertEqual(mail.outbox[1].to, [settings.BALAFON_NOTIFICATION_EMAIL])

        store_item1 = models.StoreItem.objects.get(id=store_item1.id)
        self.assertEqual(store_item1.stock_count, None)

        store_item2 = models.StoreItem.objects.get(id=store_item2.id)
        self.assertEqual(store_item2.stock_count, None)

    def test_post_cart_no_profile_full_address(self):
        """It should create a new sale and action"""

        store_item1 = mommy.make(models.StoreItem)
        store_item2 = mommy.make(models.StoreItem)

        delivery_point = mommy.make(models.DeliveryPoint)

        zone_type = ZoneType.objects.get_or_create(type="country")[0]
        default_country = get_default_country()
        country = Zone.objects.get_or_create(name=default_country, type=zone_type, parent=None)[0]

        contact_data = {
            'lastname': 'Dupond',
            'firstname': 'Paul',
            'email': 'paul@dupond.fr',
            'address': '1 rue la ré',
            'address2': 'Bat B',
            'address3': 'à gauche',
            'city': 'Lyon',
            'zip_code': '69000',
            'country': country.id,
        }

        data = {
            'items': [
                {'id': store_item1.id, 'quantity': 2},
                {'id': store_item2.id, 'quantity': 1},
            ],
            'delivery_point': delivery_point.id,
            'purchase_datetime': datetime(2015, 7, 23, 12, 0),
            'contact': contact_data,
        }

        url = reverse('store_post_cart_no_profile')

        response = self.client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(response.data['ok'], True)
        self.assertEqual(len(response.data['warnings']), 0)
        self.assertEqual(response.data['deliveryDate'], data['purchase_datetime'])
        self.assertEqual(response.data['deliveryPlace'], delivery_point.name)

        action_type = ActionType.objects.get(name=get_cart_type_name())

        action_queryset = Action.objects.filter(type=action_type)
        self.assertEqual(action_queryset.count(), 1)
        action = action_queryset[0]

        self.assertEqual(response.data['action'], action.id)

        self.assertEqual(action.contacts.count(), 1)
        contact = action.contacts.all()[0]
        self.assertEqual(contact.lastname, contact_data['lastname'])
        self.assertEqual(contact.firstname, contact_data['firstname'])
        self.assertEqual(contact.email, contact_data['email'])
        self.assertEqual(contact.address, contact_data['address'])
        self.assertEqual(contact.address2, contact_data['address2'])
        self.assertEqual(contact.address3, contact_data['address3'])
        self.assertEqual(contact.city.name, contact_data['city'])
        self.assertEqual(contact.zip_code, contact_data['zip_code'])
        self.assertEqual(contact.get_country(), country)

        self.assertEqual(action.subject, '')
        self.assertEqual(action.detail, '')
        self.assertEqual(action.planned_date, data['purchase_datetime'])
        self.assertEqual(action.sale.payment_mode, None)
        self.assertEqual(action.sale.analysis_code, models.SaleAnalysisCode.objects.get(name='Internet'))

        self.assertEqual(action.sale.saleitem_set.count(), 2)
        self.assertEqual(action.sale.saleitem_set.all()[0].item, store_item1)
        self.assertEqual(action.sale.saleitem_set.all()[0].text, store_item1.name)
        self.assertEqual(action.sale.saleitem_set.all()[0].unit_price(), store_item1.pre_tax_price)
        self.assertEqual(action.sale.saleitem_set.all()[0].vat_rate, store_item1.vat_rate)
        self.assertEqual(action.sale.saleitem_set.all()[0].quantity, 2)

        self.assertEqual(action.sale.saleitem_set.all()[1].item, store_item2)
        self.assertEqual(action.sale.saleitem_set.all()[1].text, store_item2.name)
        self.assertEqual(action.sale.saleitem_set.all()[1].unit_price(), store_item2.pre_tax_price)
        self.assertEqual(action.sale.saleitem_set.all()[1].vat_rate, store_item2.vat_rate)
        self.assertEqual(action.sale.saleitem_set.all()[1].quantity, 1)

        self.assertEqual(len(mail.outbox), 2)
        self.assertEqual(mail.outbox[0].to, [contact.email])
        self.assertEqual(mail.outbox[1].to, [settings.BALAFON_NOTIFICATION_EMAIL])

        store_item1 = models.StoreItem.objects.get(id=store_item1.id)
        self.assertEqual(store_item1.stock_count, None)

        store_item2 = models.StoreItem.objects.get(id=store_item2.id)
        self.assertEqual(store_item2.stock_count, None)

    @override_settings(BALAFON_ALLOW_CART_NO_PROFILE=False)
    def test_post_cart_no_profile_disabled(self):
        """It should raise 404"""

        store_item1 = mommy.make(models.StoreItem)
        store_item2 = mommy.make(models.StoreItem)

        delivery_point = mommy.make(models.DeliveryPoint)

        zone_type = ZoneType.objects.get_or_create(type="country")[0]
        default_country = get_default_country()
        country = Zone.objects.get_or_create(name=default_country, type=zone_type, parent=None)[0]

        contact_data = {
            'lastname': 'Dupond',
            'firstname': 'Paul',
            'email': 'paul@dupond.fr',
            'address': '1 rue la ré',
            'city': 'Lyon',
            'zip_code': '69000',
            'country': country.id,
        }

        data = {
            'items': [
                {'id': store_item1.id, 'quantity': 2},
                {'id': store_item2.id, 'quantity': 1},
            ],
            'delivery_point': delivery_point.id,
            'purchase_datetime': datetime(2015, 7, 23, 12, 0),
            'contact': contact_data,
        }

        url = reverse('store_post_cart_no_profile')

        response = self.client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
        action_queryset = Action.objects.all()
        self.assertEqual(action_queryset.count(), 0)

    def _do_test_post_cart_missing(self, extra_data):
        """It should create a new sale and action"""

        store_item1 = mommy.make(models.StoreItem)
        store_item2 = mommy.make(models.StoreItem)

        delivery_point = mommy.make(models.DeliveryPoint)

        zone_type = ZoneType.objects.get_or_create(type="country")[0]
        default_country = get_default_country()
        country = Zone.objects.get_or_create(name=default_country, type=zone_type, parent=None)[0]

        contact_data = {
            'lastname': 'Dupond',
            'firstname': 'Paul',
            'email': 'paul@dupond.fr',
            'address': '1 rue la ré',
            'city': 'Lyon',
            'zip_code': '69000',
            'country': country.id,
        }
        contact_data.update(extra_data)

        data = {
            'items': [
                {'id': store_item1.id, 'quantity': 2},
                {'id': store_item2.id, 'quantity': 1},
            ],
            'delivery_point': delivery_point.id,
            'purchase_datetime': datetime(2015, 7, 23, 12, 0),
            'contact': contact_data,
        }

        url = reverse('store_post_cart_no_profile')

        response = self.client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['ok'], False)
        action_queryset = Action.objects.all()
        self.assertEqual(action_queryset.count(), 0)
        self.assertEqual(Contact.objects.count(), 0)
        self.assertEqual(len(mail.outbox), 0)

    def test_post_cart_empty_lastname(self):
        self._do_test_post_cart_missing({'lastname': ''})

    def test_post_cart_empty_firstname(self):
        self._do_test_post_cart_missing({'firstname': ''})

    def test_post_cart_empty_email(self):
        self._do_test_post_cart_missing({'email': ''})

    def test_post_cart_invalid_email(self):
        self._do_test_post_cart_missing({'email': 'test'})

    def test_post_cart_empty_address(self):
        self._do_test_post_cart_missing({'address': ''})

    def test_post_cart_empty_zip_code(self):
        self._do_test_post_cart_missing({'zip_code': ''})

    def test_post_cart_empty_city(self):
        self._do_test_post_cart_missing({'city': ''})

    def test_post_cart_empty_country(self):
        self._do_test_post_cart_missing({'country': 0})

    def test_post_cart_invalid_zip_code(self):
        self._do_test_post_cart_missing({'zip_code': '1'})

    def test_post_cart_foreign_countru(self):

        store_item1 = mommy.make(models.StoreItem)
        store_item2 = mommy.make(models.StoreItem)

        delivery_point = mommy.make(models.DeliveryPoint)

        zone_type = ZoneType.objects.get_or_create(type="country")[0]
        default_country = get_default_country()
        country = Zone.objects.get_or_create(name=default_country, type=zone_type, parent=None)[0]
        other_country = Zone.objects.create(name='Foreign', type=zone_type, parent=None)

        contact_data = {
            'lastname': 'Dupond',
            'firstname': 'Paul',
            'email': 'paul@dupond.fr',
            'address': '1 rue la ré',
            'city': 'Foreign',
            'zip_code': 'AAZ',
            'country': other_country.id,
        }

        data = {
            'items': [
                {'id': store_item1.id, 'quantity': 2},
                {'id': store_item2.id, 'quantity': 1},
            ],
            'delivery_point': delivery_point.id,
            'purchase_datetime': datetime(2015, 7, 23, 12, 0),
            'contact': contact_data,
        }

        url = reverse('store_post_cart_no_profile')

        response = self.client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(response.data['ok'], True)
        self.assertEqual(len(response.data['warnings']), 0)
        self.assertEqual(response.data['deliveryDate'], data['purchase_datetime'])
        self.assertEqual(response.data['deliveryPlace'], delivery_point.name)

        action_type = ActionType.objects.get(name=get_cart_type_name())

        action_queryset = Action.objects.filter(type=action_type)
        self.assertEqual(action_queryset.count(), 1)
        action = action_queryset[0]

        self.assertEqual(response.data['action'], action.id)

        self.assertEqual(action.contacts.count(), 1)
        contact = action.contacts.all()[0]
        self.assertEqual(contact.lastname, contact_data['lastname'])
        self.assertEqual(contact.firstname, contact_data['firstname'])
        self.assertEqual(contact.email, contact_data['email'])
        self.assertEqual(contact.address, contact_data['address'])
        self.assertEqual(contact.city.name, contact_data['city'])
        self.assertEqual(contact.zip_code, contact_data['zip_code'])
        self.assertEqual(contact.get_country(), other_country)

        self.assertEqual(action.subject, '')
        self.assertEqual(action.detail, '')
        self.assertEqual(action.planned_date, data['purchase_datetime'])
        self.assertEqual(action.sale.payment_mode, None)
        self.assertEqual(action.sale.analysis_code, models.SaleAnalysisCode.objects.get(name='Internet'))

        self.assertEqual(action.sale.saleitem_set.count(), 2)
        self.assertEqual(action.sale.saleitem_set.all()[0].item, store_item1)
        self.assertEqual(action.sale.saleitem_set.all()[0].text, store_item1.name)
        self.assertEqual(action.sale.saleitem_set.all()[0].unit_price(), store_item1.pre_tax_price)
        self.assertEqual(action.sale.saleitem_set.all()[0].vat_rate, store_item1.vat_rate)
        self.assertEqual(action.sale.saleitem_set.all()[0].quantity, 2)

        self.assertEqual(action.sale.saleitem_set.all()[1].item, store_item2)
        self.assertEqual(action.sale.saleitem_set.all()[1].text, store_item2.name)
        self.assertEqual(action.sale.saleitem_set.all()[1].unit_price(), store_item2.pre_tax_price)
        self.assertEqual(action.sale.saleitem_set.all()[1].vat_rate, store_item2.vat_rate)
        self.assertEqual(action.sale.saleitem_set.all()[1].quantity, 1)

        self.assertEqual(len(mail.outbox), 2)
        self.assertEqual(mail.outbox[0].to, [contact.email])
        self.assertEqual(mail.outbox[1].to, [settings.BALAFON_NOTIFICATION_EMAIL])

        store_item1 = models.StoreItem.objects.get(id=store_item1.id)
        self.assertEqual(store_item1.stock_count, None)

        store_item2 = models.StoreItem.objects.get(id=store_item2.id)
        self.assertEqual(store_item2.stock_count, None)


# -*- coding: utf-8 -*-

from django.contrib import admin
from django.contrib import messages
from django.utils.translation import ugettext_lazy as _, ugettext

from . import models
from .utils import create_profile_contact


class ContactProfileCustomFieldInline(admin.TabularInline):
    model = models.ContactProfileCustomField


def finalize_user_profile(modeladmin, request, queryset):
    for profile in queryset:
        if profile.contact is None:
            create_profile_contact(profile.user)
            messages.success(request, ugettext('{0}: Contact created').format(profile.user))
        else:
            messages.warning(request, ugettext('{0}: Contact already existing').format(profile.user))


finalize_user_profile.short_description = _('Create contact')


@admin.register(models.ContactProfile)
class ContactProfileAdmin(admin.ModelAdmin):
    list_display = ['user', 'contact']
    raw_id_fields = ['user', 'contact', 'city']
    inlines = [ContactProfileCustomFieldInline]
    actions = [finalize_user_profile]


@admin.register(models.CategoryPermission)
class CategoryPermissionAdmin(admin.ModelAdmin):
    list_display = ['__str__']
    filter_horizontal = ['can_view_groups', 'can_edit_groups']



# Generated by Django 2.2.17 on 2020-12-01 15:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Store', '0021_saleitem_is_extra'),
    ]

    operations = [
        migrations.AddField(
            model_name='storemanagementactiontype',
            name='footer_template_name',
            field=models.CharField(blank=True, default='', help_text='Set the name of a custom template for commercial document footer', max_length=100, verbose_name='footer template name'),
        ),
        migrations.AddField(
            model_name='storemanagementactiontype',
            name='header_template_name',
            field=models.CharField(blank=True, default='', help_text='Set the name of a custom template for commercial document header', max_length=100, verbose_name='header template name'),
        ),
    ]



# -*- coding: utf-8 -*-
"""app configuration"""

from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class BalafonAppConfig(AppConfig):
    name = 'balafon.Store'
    verbose_name = _("Balafon Store")



# Generated by Django 2.2.17 on 2020-11-23 17:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Store', '0017_auto_20180928_1358'),
    ]

    operations = [
        migrations.AddField(
            model_name='sale',
            name='details',
            field=models.TextField(blank=True, default=''),
        ),
    ]



# Generated by Django 2.2.18 on 2021-02-23 09:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Crm', '0031_auto_20201202_1754'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='teammember',
            options={'ordering': ['ordering', 'name'], 'verbose_name': 'team member', 'verbose_name_plural': 'team members'},
        ),
        migrations.AddField(
            model_name='teammember',
            name='ordering',
            field=models.IntegerField(default=0, verbose_name='ordering'),
        ),
    ]



# -*- coding: utf-8 -*-

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Crm', '0008_auto_20160201_1656'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='sameas',
            name='main_contact',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-07-09 13:05

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('Crm', '0024_actionstatus_next_on_send'),
        ('Store', '0014_saleitem_percentage'),
    ]

    operations = [
        migrations.AddField(
            model_name='storemanagementactiontype',
            name='default_readonly_status',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='Crm.ActionStatus', verbose_name='default readonly status'),
        ),
    ]



# Generated by Django 2.2.18 on 2021-02-18 08:28

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('Store', '0022_auto_20201201_1525'),
    ]

    operations = [
        migrations.CreateModel(
            name='PaymentMode',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100, verbose_name='name')),
                ('code', models.CharField(max_length=100, verbose_name='code')),
                ('is_active', models.BooleanField(default=True, verbose_name='is active')),
            ],
            options={
                'verbose_name': 'Payment mode',
                'verbose_name_plural': 'Payment modes',
            },
        ),
        migrations.AddField(
            model_name='sale',
            name='payment_mode',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, to='Store.PaymentMode', verbose_name='payment mode'),
        ),
    ]



# -*- coding: utf-8 -*-

from django.conf import settings
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.core.mail import EmailMessage
from django.http import Http404, HttpResponseRedirect
from django.shortcuts import render
from django.template.loader import get_template, render_to_string
from django.urls import reverse
from django.utils.translation import ugettext as _

from django_registration.backends.activation.views import RegistrationView, ActivationView

from balafon.Crm.models import Action, ActionType
from balafon.Profile.forms import MessageForm
from balafon.Profile.models import ContactProfile
from balafon.Profile.settings import is_registration_enabled, is_html_activation_email
from balafon.Profile.utils import create_profile_contact, notify_registration
from balafon.settings import get_profile_form, get_registration_form
from balafon.utils import now_rounded, send_email


@login_required 
def edit_profile(request):
    try:
        profile = request.user.contactprofile
    except ContactProfile.DoesNotExist:
        raise Http404

    profile_form_class = get_profile_form()
    
    if request.method == "POST":
        form = profile_form_class(request.POST, request.FILES, instance=profile.contact)
        if form.is_valid():
            # save contact
            form.save()
            messages.add_message(request, messages.SUCCESS, _("Your profile has been updated."))
            return HttpResponseRedirect(reverse('homepage'))
    else:
        form = profile_form_class(instance=profile.contact)
        
    return render(
        request,
        'Profile/edit_profile.html',
        {
            'contact': profile.contact,
            'form': form,
        }
    )


@login_required 
def post_message(request):
    try:
        profile = request.user.contactprofile
    except ContactProfile.DoesNotExist:
        raise Http404

    if not profile.contact:
        raise Http404
    
    if request.method == "POST":
        
        form = MessageForm(request.POST)
        if form.is_valid():
            
            message = form.cleaned_data['message']
            
            # send message by email
            notification_email = getattr(settings, 'BALAFON_NOTIFICATION_EMAIL', '')
            if notification_email:
                from_email = getattr(settings, 'DEFAULT_FROM_EMAIL')
                
                data = {
                    'contact': profile.contact,
                    'message': message,
                    'site': settings.COOP_CMS_SITE_PREFIX,
                }
                template_ = get_template('Emailing/subscribe_notification_email.txt')
                content = template_.render(data)
                email = EmailMessage(
                    _("Message from web site"), content, from_email,
                    [notification_email]
                    # headers={'Reply-To': profile.contact.email}
                )
                try:
                    email.send()
                    messages.add_message(request, messages.SUCCESS, _("The message have been sent"))
                except Exception:
                    messages.add_message(request, messages.ERROR, _("The message couldn't be send."))
                    
            # add an action
            message_action, _is_new = ActionType.objects.get_or_create(name=_('Message'))
            Action.objects.create(
                subject=_("New message on web site"), planned_date=now_rounded(),
                type=message_action, detail=message, contact=profile.contact, display_on_board=True
            )
        
            return HttpResponseRedirect(reverse('homepage'))
    else:
        form = MessageForm()
        
    return render(
        request,
        'Profile/post_message.html',
        {
            'contact': profile.contact,
            'form': form,
        }
    )


class AcceptNewsletterRegistrationView(RegistrationView):
    
    def get_form_class(self, request=None):
        return get_registration_form()

    def dispatch(self, request, *args, **kwargs):
        """make possible to disable this view from settings"""
        if not is_registration_enabled():
            raise Http404
        return super(AcceptNewsletterRegistrationView, self).dispatch(request, *args, **kwargs)

    def create_inactive_user(self, form):
        """
        Create the inactive user account and send an email containing
        activation instructions.

        """
        new_user = form.save(commit=False)
        new_user.is_active = False
        new_user.save()

        return new_user

    def register(self, request_or_form, **kwargs):
        data = request_or_form.cleaned_data
        user = super(AcceptNewsletterRegistrationView, self).register(request_or_form, **kwargs)

        user.first_name = data.get('firstname', "")
        user.last_name = data.get('lastname', "")

        user.contactprofile.firstname = data.get('firstname', "")
        user.contactprofile.lastname = data.get('lastname', "")

        user.contactprofile.entity_type = data.get('entity_type', None)
        user.contactprofile.entity_name = data.get('entity', "")

        user.contactprofile.phone = data.get('phone', "")
        user.contactprofile.mobile = data.get('mobile', "")

        user.contactprofile.city = data.get('city', None)
        user.contactprofile.zip_code = data.get('zip_code', None)
        user.contactprofile.gender = data.get('gender', 0) or 0

        user.contactprofile.address = data.get('address', "")
        user.contactprofile.address2 = data.get('address2', "")
        user.contactprofile.address3 = data.get('address3', "")
        user.contactprofile.cedex = data.get('cedex', "")
        user.contactprofile.birth_date = data.get('birth_date', None)

        user.contactprofile.groups_ids = data.get('groups_ids', '')

        user.save()
        user.contactprofile.save()

        subscription_types = data.get('subscription_types', None)

        user.contactprofile.subscriptions_ids = ",".join([str(s.id) for s in subscription_types])
        user.contactprofile.save()

        self.send_activation_email(user)

        return user

    def send_activation_email(self, user):
        if not is_html_activation_email():
            super(AcceptNewsletterRegistrationView, self).send_activation_email(user)
        else:
            activation_key = self.get_activation_key(user)
            context = self.get_email_context(activation_key)
            context['user'] = user
            context['profile'] = user.contactprofile
            subject = render_to_string(
                template_name=self.email_subject_template,
                context=context,
                request=self.request
            )
            # Force subject to a single line to avoid header-injection issues.
            # send an email
            subject = ''.join(subject.splitlines())
            send_email(subject, 'Profile/activation_email_body.html', context, [user.email])


class AcceptNewsletterActivationView(ActivationView):

    def activate(self, request, *args, **kwargs):
        activated_user = super(AcceptNewsletterActivationView, self).activate(request, *args, **kwargs)
        # The account has been activated: We can create the corresponding contact in Balafon
        if activated_user:
            profile = create_profile_contact(activated_user)
            notify_registration(profile)
        return activated_user



# -*- coding: utf-8 -*-

from django.conf import settings
from django.contrib.auth.models import User
from django.db import models
from django.db.models import signals
from django.utils.translation import ugettext_lazy as _

from coop_cms.models import ArticleCategory

from balafon.Crm.models import Contact, Group, City, EntityType


class ContactProfile(models.Model):
    """Info creating a contact from registration"""

    class Meta:
        verbose_name = _('Contact profile')
        verbose_name_plural = _('Contact profiles')
    
    GENDER_CHOICE = (
        (Contact.GENDER_NOT_SET, ''),
        (Contact.GENDER_MALE, _('Mr')),
        (Contact.GENDER_FEMALE, _('Mrs')),
    )

    user = models.OneToOneField(User, on_delete=models.CASCADE)

    contact = models.OneToOneField(Contact, blank=True, default=None, null=True, on_delete=models.CASCADE)

    entity_name = models.CharField(_('Entity name'), max_length=200, blank=True, default="")
    entity_type = models.ForeignKey(
        EntityType, verbose_name=_('Entity type'), blank=True, null=True, default=None, on_delete=models.CASCADE
    )

    zip_code = models.CharField(_('Zip code'), max_length=20, blank=True, default='')
    city = models.ForeignKey(
        City, verbose_name=_('City'), blank=True, default=None, null=True, on_delete=models.CASCADE
    )
    gender = models.IntegerField(_('Gender'), choices=GENDER_CHOICE, blank=True, default=0)
    lastname = models.CharField(_('last name'), max_length=200, blank=True, default='')
    firstname = models.CharField(_('first name'), max_length=200, blank=True, default='')

    birth_date = models.DateField(_("birth date"), blank=True, default=None, null=True)

    phone = models.CharField(_('phone'), max_length=200, blank=True, default='')
    mobile = models.CharField(_('mobile'), max_length=200, blank=True, default='')

    address = models.CharField(_('address'), max_length=200, blank=True, default='')
    address2 = models.CharField(_('address 2'), max_length=200, blank=True, default='')
    address3 = models.CharField(_('address 3'), max_length=200, blank=True, default='')
    cedex = models.CharField(_('cedex'), max_length=200, blank=True, default='')

    subscriptions_ids = models.CharField(max_length=100, default="", blank=True)

    groups_ids = models.CharField(max_length=100, default="", blank=True)

    def __str__(self):
        return self.user.username

    @property
    def fullname(self):
        return '{0} {1}'.format(self.firstname, self.lastname)


class ContactProfileCustomField(models.Model):
    profile = models.ForeignKey(ContactProfile, on_delete=models.CASCADE, related_name='custom_fields')
    name = models.CharField(_('name'), max_length=100)
    value = models.CharField(_('value'), max_length=200, blank=True, default='')
    entity_field = models.BooleanField(default=False)

 
# signals
def create_profile(sender, instance, signal, created, **kwargs):
    if not created:
        created_profile = False
        try:
            ContactProfile.objects.get(user=instance)
        except ContactProfile.DoesNotExist:
            created_profile = True
    else:
        created_profile = True
        
    if created_profile:
        ContactProfile(user=instance).save()


if "balafon.Profile" in settings.INSTALLED_APPS:
    signals.post_save.connect(create_profile, sender=User)


class CategoryPermission(models.Model):
    """Permissions on category"""

    class Meta:
        verbose_name = _('Category permission')
        verbose_name_plural = _('Category permissions')

    category = models.OneToOneField(ArticleCategory, on_delete=models.CASCADE)
    can_view_groups = models.ManyToManyField(
        Group, blank=True, default=None, related_name="can_view_perm"
    )
    can_edit_groups = models.ManyToManyField(
        Group, blank=True, default=None, related_name="can_edit_perm"
    )
    
    def __str__(self):
        return '{0}'.format(self.category)



# Generated by Django 2.2.18 on 2021-02-23 09:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Store', '0023_auto_20210218_0828'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='paymentmode',
            options={'ordering': ['ordering', 'id'], 'verbose_name': 'Payment mode', 'verbose_name_plural': 'Payment modes'},
        ),
        migrations.AddField(
            model_name='paymentmode',
            name='ordering',
            field=models.IntegerField(default=0, verbose_name='ordering'),
        ),
    ]



# -*- coding: utf-8 -*-
"""test create newsletter"""

from django.contrib.sites.models import Site
from django.urls import reverse

from model_mommy import mommy
from coop_cms.models import Newsletter
from coop_cms.settings import get_newsletter_templates

from balafon.Emailing.tests import BaseTestCase


class AddNewsletterTest(BaseTestCase):

    def test_view_newsletter(self):
        url = reverse("emailing_new_newsletter")
        site1 = Site.objects.get_current()
        site2 = mommy.make(Site)
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(Newsletter.objects.count(), 0)

    def test_create_newsletter(self):
        url = reverse("emailing_new_newsletter")
        site1 = Site.objects.get_current()
        site2 = mommy.make(Site)
        templates = get_newsletter_templates(None, None)
        data = {
            "subject": "test",
            "template": templates[0][0],
            'site': site2.id,
        }
        response = self.client.post(url, data=data)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(Newsletter.objects.count(), 1)
        newsletter = Newsletter.objects.all()[0]
        self.assertEqual(newsletter.subject, data["subject"])
        self.assertEqual(newsletter.template, data["template"])
        self.assertEqual(newsletter.site, site2)



# Generated by Django 2.2.14 on 2020-12-15 10:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Users', '0006_auto_20200210_1436'),
    ]

    operations = [
        migrations.AddField(
            model_name='custommenuitem',
            name='query_string',
            field=models.CharField(blank=True, default='', max_length=100, verbose_name='query string'),
        ),
    ]
